/*
восприятие действий и фраз с Пусльта

Создание образов различной иерархии контекстов восприятия:
BaseStyleArr - образ сочетаний активных Базовых контекстов
TriggerStimulsArr - образ сочетаний пусковых стимулов

Создание образов сочетаний ID действий:
*/

package reflexes

import (
	"BOT/brain/gomeostas"
	"BOT/brain/psychic"
	"BOT/brain/sleep"
)

///////////////////////////////////////
func loadImagesArrs(){
	//  загрузить образы сочетаний базовых стилей
	loadBaseStyleArr()
	//  загрузить образы сочетаний пусковых стимулов
	loadTriggerStimulsArr()

}
/////////////////////////////////////////////
// ПУЛЬС рефлексов
var ReflexPulsCount=0 // передача тика Пульса из brine.go
var LifeTime=0
var EvolushnStage=0  // стадия развития
var IsSlipping=false
// коррекция текущего состояния гомеостаза и базового контекста с каждым пульсом
func ReflexCountPuls(evolushnStage int,lifeTime int,puls int,isSlipping bool){
	LifeTime=lifeTime
	EvolushnStage=evolushnStage
	ReflexPulsCount=puls // передача номера тика из более низкоуровневого пакета
	IsSlipping=isSlipping

	if gomeostas.IsNewConditions{// изменились условия
		ActiveFromConditionChange()
		return
	}

	if WakeUpping{
		sleep.WakeUpping()
	}

	// обнулить причину возможного запуска рефлекса
	if oldActiveCurTriggerStimulsID>0 && oldActiveCurTriggerStimulsPulsCount>(ReflexPulsCount+10){
		oldActiveCurTriggerStimulsID=0
	}
}
//////////////////////////////////////////////////////////////







//////  АКТИВАЦИЯ ДЕРЕВА РЕФЛЕКСОВ ПО изменению условий, действиям с Пульта или фразе с Пульта

/*  Вид активации дерева рефлексов:
1 - изменение сочетания базовых контекстов
2 - действия с Пульта
3 - фраза с Пульта
 */
var ActivationTypeSensor=0


////////////////////////////////////
// текущее восприятие ID образов
//обновляющихся при каждом событии с Пульта или достаточно сильном изменении Базовых параметров
var ActiveCurBaseID=0 // ID Базового состояния CommonBadNormalWell
var ActiveCuraseStyleID=0 // ID сочетания базовых контекстов BaseStyle
var ActiveCurTriggerStimulsID=0 // ID теущего активного образа сочетаний пусковых стимулов TriggerStimuls
/* предыдущий образ сочетания пусковых стимулов
используется как ПРИЧИНА последующих событий - для формирования условных рефлексов
потому как ОБНУЛЯЕТСЯ при:
1) активации дерева рефлексов, если вызвало какое-то действие
2) через 10 пульсов после записи значения - типа причина устаревает
 */
var oldActiveCurTriggerStimulsID=0
func setOldActiveCurTriggerStimulsVal(val int){
	oldActiveCurTriggerStimulsID=val
	oldActiveCurTriggerStimulsPulsCount=ReflexPulsCount
}
// момент записи значения в тике Пульса
var oldActiveCurTriggerStimulsPulsCount=0

////////////////////////
/// Дерево рефлексов активируется при любом изменении условий с проверкой по каждому пульсу:
func ActiveFromConditionChange(){
	ActivationTypeSensor=1
	// если при активации уровня психики сработает автоматизм или будет произвольная блокировка рефлексов
	if psychic.SensorActivation(ActivationTypeSensor){
		return
	}

	ActiveCurBaseID=gomeostas.CommonBadNormalWell
	// определение текущего сочетания ID Базовых контекстов
	bsIDarr:=gomeostas.GetCurContextActiveIDarr()

	// создаем новый образ Базовых контекстов, если такого еще нет
	ActiveCuraseStyleID,_=createNewBaseStyle(0,bsIDarr)

	// активировать дерево
	activeReflexTree()
}
////////////////////////
// активировать дерево действием reflexes.ActiveFromAction()
func ActiveFromAction(){
	ActivationTypeSensor=2
	// если при активации уровня психики сработает автоматизм или будет произвольная блокировка рефлексов
	if psychic.SensorActivation(ActivationTypeSensor){
		return
	}

	ActiveCurBaseID=gomeostas.CommonBadNormalWell
	// определение текущего сочетания ID Базовых контекстов
	bsIDarr:=gomeostas.GetCurContextActiveIDarr()

	// создаем новый образ Базовых контекстов, если такого еще нет
	ActiveCuraseStyleID,_=createNewBaseStyle(0,bsIDarr)

	// создаем новый образ Базовых контекстов, если такого еще нет
	CreateNewTriggerStimulsImage()

	// активировать дерево
	activeReflexTree()

}
////////////////////////
// активировать дерево фразой  reflexes.ActiveFromPhrase() - только для условных рефлексов
func ActiveFromPhrase(){
	ActivationTypeSensor=3
	// если при активации уровня психики сработает автоматизм или будет произвольная блокировка рефлексов
	if psychic.SensorActivation(ActivationTypeSensor){
		return
	}

	ActiveCurBaseID=gomeostas.CommonBadNormalWell
	// определение текущего сочетания ID Базовых контекстов
	bsIDarr:=gomeostas.GetCurContextActiveIDarr()

	// создаем новый образ Базовых контекстов, если такого еще нет
	ActiveCuraseStyleID,_=createNewBaseStyle(0,bsIDarr)

	// создать новое сочетание пусковых стимулов если такого еще нет
	CreateNewTriggerStimulsImage()

	// активировать дерево
	//BS:=gomeostas.GetCurContextActiveIDarr() // текущий стиль Бозовых контекстов
	activeReflexTree()
}
/////////////////////////////////////////////////////////




//////////////////////////////////////////////////////////////
/* создание иерархии образов контекстов условий и пусковых стимулов в виде ID образов в [3]int
 создать последовательность уровней условий в виде массива  ID последовательности ID уровней
В случае отсуствия пусковых стимулов создается ID такого отсутсвия, пример такой записи: 2|||0|0|
 */
func getConditionsArr(lev1ID int, lev2 []int, lev3 []int, PhraseID []int,ToneID int,MoodID int )([]int){
	arr:=make([]int, 3)
	arr[0]=lev1ID
	arr[1],_=createNewBaseStyle(0,lev2)
	arr[2],_=createNewlastTriggerStimulsID(0,lev3,PhraseID,ToneID,MoodID)
	return arr
}
////////////////////////////////////////////////////
